<?php

/*
	University of Illinois/NCSA Open Source License for cmlist.php.

Copyright (c) 2008 Jim Champ/Surrey County Council 
All rights reserved.
Developed by: 		Jim Champ
                    www.champwilde.f9.co.uk/idmphp/cmexport/
Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the
"Software"), to deal with the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimers. 

Redistributions in binary form must reproduce the above
copyright notice, this list of conditions and the following disclaimers
in the documentation and/or other materials provided with the
distribution. 

Neither the names of Surrey County Council nor the names of its contributors may be used to endorse
or promote products derived from this Software without specific prior
written permission. 
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR
ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE. 
*/

// cmlist php script to extract data from Call manager or other LDAP directory that won't
// run a proper IDM Ldap driver so that the data can be imported with a delimited text driver
// this is simply a batch export of the whole directory. 

// you probably want to run this on a daily or better basis from a cron script at the server console
// it's designed to be run from the console, and most definitely NOT as a web page!

// Set the key variables here...
// "cn=directory manager,o=cisco.com" seems to be the default account
$ldap_id = "cn=directory manager,o=cisco.com" ;

// password on the account. Default for directory manager seems to be none!
$ldap_pw = "";

// this is the ip address of the actual call manager server, no reason why this
// shouldn't be a DNS name
$ldap_server = "10.0.0.1";

// Start the search at LDAP root. Seems to be "ou=users,o=cisco.com" in a default install
$ldap_root = "ou=users,o=cisco.com";

// if you don't limit the search to inetorgperson you get all sorts of personal phone directory entries
// and the like which aren't very useful!
$ldap_sr = "objectClass=inetOrgPerson";

// default LDAP port for Call Manager is 8404
$ldapport=8404;

// output path. This must (of course) match the location for incoming documents 
// set in your delimited text driver
$outPutPath = "sys:callmgrlist.csv";

// basic sequence with LDAP is connect, bind, search, interpret search
// result, close connection

// So we start by connecting to LDAP

// first a confidence display message on the console
echo "\n";
echo "Running cmlist PHP Script : ";

// log the date to screen...
$today = getdate();
// you may wish to switch this around if your location doesn't use dd/mm/yy date formats
echo $today['hours'] . ":" . str_pad($today['minutes'], 2, "0", STR_PAD_LEFT) . ", " . $today['mday'] . "/" . $today['mon'] . "/" . $today['year'];
echo "\n";

echo "Connecting to LDAP server...\n";

$ds=ldap_connect("$ldap_server", $ldapport);  // must be a valid LDAP server!

if(@ldap_bind($ds, $ldap_id,$ldap_pw)) {
	echo "Connection succeeded.\n";
	// we have now made an LDAP connection
} else { 
	echo " LDAP Authentication Failed\n";
	echo "User id and/or Password credentials were incorrect.\n";
	exit;
}

echo "Running the LDAP Search...\n";

// do the search 
// 21 May 09, Added array of ldap attrs to the ldap_search call. This improves performance
// as only the names attributes are returned. Ensure all attributes you wish to read are
// listed in this array. Note that the attrbiute names are case sensitive.
$sr=@ldap_search($ds, $ldap_root, $ldap_sr , array(cn, givenName, sn, telephoneNumber));  

// then get the first entry of the returned data
$entry = @ldap_first_entry($ds, $sr);

// and list the no of entries returned as a progress item
echo  @ldap_count_entries($ds, $sr) . " directory entries returned.\n";

// kick off the output file
@$fHandle = fopen($outPutPath,"w");

// now iterate through the search results
while ($entry) {
	// null the output variable
	$contents ="";

	// get attributes of the current DN
	$attrs = @ldap_get_attributes($ds,$entry);
	
	// display CN on screen...
	echo @$attrs["cn"][0];
	echo "\n";
	
	// echo and now push the other desired attributes into the 
	// big variable with the output
	$contents .= "\"";
	
	// in case of multi=valued attrs we just grab the first one
	// doesn't seem to happen in our Call Manager directory
	$contents .=  @$attrs["cn"][0];
	$contents .= "\",\"";
	$contents .=  @$attrs["givenName"][0];
	$contents .= "\",\"";
	$contents .=  @$attrs["sn"][0] ;
	$contents .= "\",\"";
	$contents .=  @$attrs["telephoneNumber"][0];
	$contents .= "\"\r\n";
	
	// cn, given name and surname are probably the most useful to be able
	// use to match in your delimited text driver
	// but you can add any other LDAP attribute here
	// use something like softterra LDAp browser to check the attr name and contents
	// note that attribute names in LDAP are case sensitive
	// $contents .=  @$attrs["mail"][0];
	// $contents .= "\",\"";
	
	// write out the current data...
	@fwrite($fHandle,$contents);
	
	// and read the next entry
	$entry = @ldap_next_entry($ds, $entry);
}

// do the housekeeping
ldap_close($ds);
fclose($fHandle);

// and notify completion
echo "cmlist PHP Script completed.\n\n";
exit();
?>
